<?php declare(strict_types=1);


namespace Pinpoint\DropShipping\Controller\Adminhtml\Supplier;


class Edit extends \Pinpoint\DropShipping\Controller\Adminhtml\Supplier
{

    protected $resultPageFactory;

    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\Registry $coreRegistry
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\Registry $coreRegistry,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory
    ) {
        $this->resultPageFactory = $resultPageFactory;
        parent::__construct($context, $coreRegistry);
    }

    /**
     * Edit action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        // 1. Get ID and create model
        $id = $this->getRequest()->getParam('supplier_id');
        $model = $this->_objectManager->create(\Pinpoint\DropShipping\Model\Supplier::class);
        
        // 2. Initial checking
        if ($id) {
            $model->load($id);
            if (!$model->getId()) {
                $this->messageManager->addErrorMessage(__('This Supplier no longer exists.'));
                /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
                $resultRedirect = $this->resultRedirectFactory->create();
                return $resultRedirect->setPath('*/*/');
            }
        }
        $this->_coreRegistry->register('pinpoint_dropshipping_supplier', $model);
        
        // 3. Build edit form
        /** @var \Magento\Backend\Model\View\Result\Page $resultPage */
        $resultPage = $this->resultPageFactory->create();
        $this->initPage($resultPage)->addBreadcrumb(
            $id ? __('Edit Supplier') : __('New Supplier'),
            $id ? __('Edit Supplier') : __('New Supplier')
        );
        $resultPage->getConfig()->getTitle()->prepend(__('Suppliers'));
        $resultPage->getConfig()->getTitle()->prepend($model->getId() ? __('Edit Supplier %1', $model->getId()) : __('New Supplier'));
        return $resultPage;
    }
}

