<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * @category    Pinpoint
 * @copyright   Copyright (c) 2020 Pinpoint Designs. (http://www.pinpointdesigns.co.uk)
 * @author      Daniel Cuevas <daniel@pinpointdesigns.co.uk>
 */

namespace Pinpoint\DropShipping\ViewModel;

use Magento\Backend\Model\Url;
use Magento\Sales\Model\OrderRepository;
use Pinpoint\DropShipping\Model\Product\Attribute\Source\DefaultSupplier;
use Pinpoint\DropShipping\Model\ResourceModel\Supplier\CollectionFactory;

/**
 * Class DropshipItem
 */
class DropshipItem  implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    /** @var \Magento\Framework\Registry $registry Registry */
    private $registry;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    /**
     * @var CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var \Magento\Framework\App\Request\Http $request
     */
    private $request;
    /**
     * @var DefaultSupplier
     */
    private $supplierSource;
    /**
     * @var Url
     */
    private $url;
    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\Item\CollectionFactory
     */
    private $orderItemColFactory;

    /**
     * DropshipItem constructor.
     * @param \Magento\Framework\Registry $registry
     */
    public function __construct(
        \Magento\Framework\Registry $registry,
        OrderRepository $orderRepository,
        \Magento\Sales\Model\ResourceModel\Order\Item\CollectionFactory $orderItemColFactory,
        \Magento\Framework\App\Request\Http $request,
        Url $url,
        DefaultSupplier $defaultSupplier,
        CollectionFactory $collectionFactory)
    {
        $this->registry = $registry;
        $this->orderRepository = $orderRepository;
        $this->collectionFactory = $collectionFactory;
        $this->request = $request;
        $this->url = $url;
        $this->supplierSource = $defaultSupplier;
        $this->orderItemColFactory = $orderItemColFactory;
    }

    public function getOrderItems()
    {
        $order = $this->getCurrentOrder();
        $orderItemCollection = $this->orderItemColFactory->create()
            ->addFieldToFilter('main_table.order_id', $order->getId())
            ->addFieldToFilter('parent_item_id', ['null' => true])
            ->addFieldToFilter('dropshipitem_id', ['null' => true]);

        $orderItemCollection->getSelect()->joinLeft(['di' => 'pinpoint_dropshipping_dropshipitem'], 'di.item_id=main_table.item_id', []);

        return $orderItemCollection;
    }

    public function getCurrentOrder()
    {
        return $this->registry->registry('current_order');
    }

    public function isDefaultSupplier($orderItem, $supplierId)
    {
        return ($orderItem->getProduct()->getSupplier() === $supplierId);
    }

    public function getSuppliers()
    {
        return $this->supplierSource->getAllOptions();
    }

    public function getDropshipUrl()
    {
        return $this->url->getUrl('pinpoint_dropshipping/dropship/massDropship');
    }
}
